import logging
from typing import List, Dict

from sqlalchemy.exc import IntegrityError, SQLAlchemyError, DatabaseError as SqlAlchemyDatabaseError

from utils.errors.database import DatabaseError, DuplicateEntryError, DatabaseConnectionError
from models import Member
from database.manager import DatabaseManager

logger = logging.getLogger(__name__)

def fetch_all_members() -> List[Member]:
    """
    Fetches all members from the database.

    :return: A list of all members in the database.
    :raises DatabaseConnectionError: If the connection to the database is interrupted.
    :raises DatabaseError: If any other error occurs while fetching members.
    """
    with DatabaseManager.get_session() as session:
        try:
            return session.query(Member).all()
        except SqlAlchemyDatabaseError as e:
            logger.critical("Connection with database interrupted")
            raise DatabaseConnectionError("Connection with database interrupted") from e
        except SQLAlchemyError as e:
            logger.error(f"An error occurred when fetching all members: {e}")
            raise DatabaseError("An error occurred when fetching all members") from e

def create_member(new_member: Dict[str, str]):
    create_members([new_member])

def create_members(members: List[Dict[str, str]]):
    try:
        with DatabaseManager.get_session() as session:
            session.add_all(members)
            session.commit()
    except IntegrityError as e:
        session.rollback()
        logger.warning("Data already exists")
        raise DuplicateEntryError("Data already exists in the database") from e
    except DatabaseError as e:
        session.rollback()
        logger.critical("Connection with database interrupted")
        raise DatabaseConnectionError("Connection with database interrupted") from e
    except SQLAlchemyError as e:
        session.rollback()
        logger.error(f"An error occurred when saving member: {e}")
        raise DatabaseError("An error occurred when creating a new member") from e

def update_member(member: Dict[str, str]):
    pass

__all__ = ["create_member", "create_members", "fetch_all_members"]
